use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.004003
eval "use Test::Spelling 0.12; use Pod::Wordlist::hanekomu; 1" or die $@;

add_stopwords(<DATA>);
all_pod_files_spelling_ok(qw( bin lib  ));
__DATA__
ESC
Epson
baudrate
heatInterval
heatTime
heatingDots
inkjet
msg
nH
nL
refactor
usb
atttt
bhadoria
Shantanu
Bhadoria
shantanu
at
cpan
dott
org
lib
Printer
Thermal
